@@ -43,8 +43,8 @@ def simulate_bias_field(domain_image,
     >>> log_field = simulate_bias_field(image, number_of_points=10, sd_bias_field=1.0,
     ...    number_of_fitting_levels=2, mesh_size=10)
     >>> log_field = log_field.iMath("Normalize")
-    >>> log_field_array = np.power(np.exp(log_field.numpy()), 4)
-    >>> image = image * ants.from_numpy(log_field_array, origin=image.origin,
+    >>> field_array = np.power(np.exp(log_field.numpy()), 4)
+    >>> image = image * ants.from_numpy(field_array, origin=image.origin,
     ...    spacing=image.spacing, direction=image.direction)
     """
 
@@ -75,4 +75,4 @@ def simulate_bias_field(domain_image,
 
     ants.set_direction(bias_field, direction)
 
-    return bias_field
\ No newline at end of file
+    return bias_field
